/**
 * Copyright (c) 2014 - 2019, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 *
 * @defgroup blinky_example_main main.c
 * @{
 * @ingroup blinky_example
 * @brief Blinky Example Application main file.
 *
 * This file contains the source code for a sample application to blink LEDs.
 *
 */

#include <stdbool.h>
#include <stdint.h>
#include "nrf_gpio.h"


void InitPortConfig(void);
unsigned char CheckButtonState(unsigned char*);
void UpdateButtonHistory(unsigned char*,unsigned char);
void ToggleLEDState(unsigned char *LEDstate,unsigned char i);


enum LEDNUMBERS
{
  LED1 = 13,
  LED2,
  LED3,
  LED4
};
enum BUTTONNUMBERS
{
  BUTTON1 = 11,
  BUTTON2,
  BUTTON3 = 24,
  BUTTON4
};


int main(void)
{
  unsigned char tmp;
  unsigned char ButtonStates[4] = {1,1,1,1};
  unsigned char LEDStates[4] = {1,1,1,1};
  unsigned char BS_history[4][3] = {{1,1,1},{1,1,1},{1,1,1},{1,1,1}};
  unsigned char i;

  /* Initialize */
  InitPortConfig();

  /* Main loop */
  while(true)
  {
    for(i=0;i<4;i++)
    {
      tmp = ButtonStates[i];

      UpdateButtonHistory(BS_history[i],i);
    
      if(CheckButtonState(BS_history[i]) == 1)
      {
        ButtonStates[i] = BS_history[i][2];
      }
      if(ButtonStates[i] == 0 && ButtonStates[i] != tmp)
      {
        ToggleLEDState(&LEDStates[i],i);
      }
    }
  }
}



void InitPortConfig(void)
{

  nrf_gpio_cfg_input(NRF_GPIO_PIN_MAP(0,BUTTON1),NRF_GPIO_PIN_PULLUP);
  nrf_gpio_cfg_input(NRF_GPIO_PIN_MAP(0,BUTTON2),NRF_GPIO_PIN_PULLUP);
  nrf_gpio_cfg_input(NRF_GPIO_PIN_MAP(0,BUTTON3),NRF_GPIO_PIN_PULLUP);
  nrf_gpio_cfg_input(NRF_GPIO_PIN_MAP(0,BUTTON4),NRF_GPIO_PIN_PULLUP);

  nrf_gpio_cfg_output(NRF_GPIO_PIN_MAP(0,LED1));
  nrf_gpio_pin_write(NRF_GPIO_PIN_MAP(0,LED1),1);
  nrf_gpio_cfg_output(NRF_GPIO_PIN_MAP(0,LED2));
  nrf_gpio_pin_write(NRF_GPIO_PIN_MAP(0,LED2),1);
  nrf_gpio_cfg_output(NRF_GPIO_PIN_MAP(0,LED3));
  nrf_gpio_pin_write(NRF_GPIO_PIN_MAP(0,LED3),1);
  nrf_gpio_cfg_output(NRF_GPIO_PIN_MAP(0,LED4));
  nrf_gpio_pin_write(NRF_GPIO_PIN_MAP(0,LED4),1);
}


unsigned char CheckButtonState(unsigned char BS_history[])
{
  unsigned char ret = 1;

  for(S8 i = 1; i < 3; i++)
  {
    if(BS_history[i-1] != BS_history[i])
    {
      ret = 0;
      break;
    }
    else
    {
      continue;
    }
  }

  return ret;
}

void UpdateButtonHistory(unsigned char *BS_history,unsigned char i)
{
  unsigned char NowState;
  unsigned char j;
  U8 ButtonPort;
  switch(i)
  {
    case 0:
      ButtonPort = BUTTON1;
      break;
    case 1:
      ButtonPort = BUTTON2;      
      break;  
    case 2:
      ButtonPort = BUTTON3;      
      break;  
    case 3:
      ButtonPort = BUTTON4;      
      break;  
  }

  NowState = nrf_gpio_pin_read(NRF_GPIO_PIN_MAP(0,ButtonPort));
  for(j=0;j<2;j++)
  {
    BS_history[j] = BS_history[j+1];
  }
  BS_history[j] =  NowState; 
 
}

void ToggleLEDState(unsigned char *LEDstate,unsigned char i){
  *LEDstate = !(*LEDstate);
  nrf_gpio_pin_write(NRF_GPIO_PIN_MAP(0,LED1+i),*LEDstate);

}

